"""Custom JARVIS client."""
import json
import traceback
from datetime import datetime

from aiohttp import ClientSession
from dis_snek import Snake, listen
from dis_snek.models.discord.channel import DMChannel
from dis_snek.models.discord.embed import EmbedField
from dis_snek.models.snek.context import Context, InteractionContext
from jarvis_core.db import q
from jarvis_core.db.models import Setting
from jarvis_core.util import build_embed
from jarvis_core.util.ansi import RESET, Fore, Format, fmt

DEFAULT_GUILD = 862402786116763668
DEFAULT_ERROR_CHANNEL = 943395824560394250
DEFAULT_URL = "https://paste.zevs.me/documents"

ERROR_MSG = """
Command Information:
  Name: {invoked_name}
  Args:
{arg_str}

Callback:
  Args:
{callback_args}
  Kwargs:
{callback_kwargs}
"""

KEY_FMT = fmt(Fore.GRAY)
VAL_FMT = fmt(Fore.WHITE)
CMD_FMT = fmt(Fore.GREEN, Format.BOLD)


class Jarvis(Snake):
    @listen()
    async def on_ready(self) -> None:
        """Lepton on_ready override."""
        print("Logged in as {}".format(self.user))  # noqa: T001
        print("Connected to {} guild(s)".format(len(self.guilds)))  # noqa: T001
        print(  # noqa: T001
            "https://discord.com/api/oauth2/authorize?client_id="
            "{}&permissions=8&scope=bot%20applications.commands".format(self.user.id)
        )

    async def on_command(self, ctx: InteractionContext) -> None:
        """Lepton on_command override."""
        if not isinstance(ctx.channel, DMChannel) and ctx.invoked_name not in ["pw"]:
            modlog = await Setting.find_one(q(guild=ctx.guild.id, setting="modlog"))
            if modlog:
                channel = await ctx.guild.fetch_channel(modlog.value)
                args = " ".join(f"{KEY_FMT}{k}:{VAL_FMT}{v}{RESET}" for k, v in ctx.kwargs.items())
                fields = [
                    EmbedField(
                        name="Command",
                        value=f"```ansi\n{CMD_FMT}{ctx.invoked_name}{RESET} {args}\n```",
                        inline=False,
                    ),
                ]
                embed = build_embed(
                    title="Command Invoked",
                    description=f"{ctx.author.mention} invoked a command",
                    fields=fields,
                    color="#fc9e3f",
                )
                embed.set_author(name=ctx.author.username, icon_url=ctx.author.display_avatar.url)
                embed.set_footer(
                    text=f"{ctx.author.username}#{ctx.author.discriminator} | {ctx.author.id}"
                )
                await channel.send(embed=embed)

    async def on_command_error(
        self, ctx: Context, error: Exception, *args: list, **kwargs: dict
    ) -> None:
        """Lepton on_command_error override."""
        guild = await self.fetch_guild(DEFAULT_GUILD)
        channel = await guild.fetch_channel(DEFAULT_ERROR_CHANNEL)
        error_time = datetime.utcnow().strftime("%d-%m-%Y %H:%M-%S.%f UTC")
        timestamp = int(datetime.now().timestamp())
        timestamp = f"<t:{timestamp}:T>"
        arg_str = (
            "\n".join(f"    {k}: {v}" for k, v in ctx.kwargs.items()) if ctx.kwargs else "    None"
        )
        callback_args = "\n".join(f"    - {i}" for i in args) if args else "    None"
        callback_kwargs = (
            "\n".join(f"    {k}: {v}" for k, v in kwargs.items()) if kwargs else "    None"
        )
        full_message = ERROR_MSG.format(
            error_time=error_time,
            invoked_name=ctx.invoked_name,
            arg_str=arg_str,
            callback_args=callback_args,
            callback_kwargs=callback_kwargs,
        )
        if len(full_message) >= 1900:
            error_message = "  ".join(traceback.format_exception(error))
            full_message += "Exception: |\n  " + error_message
            async with ClientSession() as session:
                resp = await session.post(DEFAULT_URL, data=full_message)
                data = await resp.read()
                data = json.loads(data.decode("UTF8"))

            await channel.send(
                f"JARVIS encountered an error at {timestamp}. Log too big to send over Discord."
                f"\nPlease see log at https://paste.zevs.me/{data['key']}"
            )
        else:
            error_message = "".join(traceback.format_exception(error))
            await channel.send(
                f"JARVIS encountered an error at {timestamp}:"
                f"\n```yaml\n{full_message}\n```"
                f"\nException:\n```py\n{error_message}\n```"
            )
        await ctx.send("Whoops! Encountered an error. The error has been logged.", ephemeral=True)
        return await super().on_command_error(ctx, error, *args, **kwargs)
